/**************************************************************

   Program Title:  LED8 Controller
   
   Program Name:  LED8_Controller.c	  Version: 0.85
   
   Author:  SS	  					  Date:  October 10, 2006
   
   Compiled with ImageCraft ICCAVR Standard V7.13A
   
   The Led8 is a set of functions that configure Led AVR internal
   peripherals for interrupt driven operation.  Custom applications can easily
   be built on top to these basic functions.  Included is a complete data 
   acquistion controller with user preferences loaded into EEPROM.  The Monitor
   communicates with the user via an RS-232 port using the CDP.   The AVR is 
   a smart powerful 8 bit RISC processor that can be easily programmed in C
   with on chip TWI, SPI and analog and digital IO.  Functions are included 
   to read/write data blocks to the on board Serial Flash.  On board sensors
   (if included) include a dual axis 2G accelerometer, temperature and light
   sensor and battery voltage monitor.
   
   Set Fuse bits as:  FEDFE2
   
   Key functions:
   	   SPI Master with read/write function to serial flash
	   TWI 2-Wire Master Interrupt Controller
	   Real Time Clock setup with four configurable software timers
	   Full duplex interrupt drive USART with XON/XOFF flow control
	   1msec timer with four configurable software times
	   Analog controller setup
	   Digital port configuration
	   Accelerometer reader
	   Temperature reader
	   Light level reader
	   IntelHex32 download/upload for EEPROM, AVRFLASH and Serial Flash
	   Auto clock calibration
	   
   Standard printf and scanf functions are not used in this version 
   due to code size constraints.  The Wasp is available with an ATmega32
   with twice the Flash and SRAM if printf and scanf functionality is 
   important or a custom application requires more than the available
   AVR Flash.
   
   Copyright 2004-2006, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2008, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC MACHINES, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: Led8_Monitor.c,v $
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, Serial Flash writer, XON/XOFF, 9,600baud
 *    real time clock and auto-internal osc calibration.
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 * 
 */ 
#include <iom168v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include "LED8_Uart.h"
#include "LED8_Defines.h"
#include "LED8_Extern.h"
#include "TWI_slave.h"


//  Variable declarations
unsigned int intelsuperaddress,oldpage,oldbyteoffset,totalrxcnt;
unsigned int rtccountdown1,rtccountdown2,rtccountdown3,rtccountdown4;
unsigned int mseccountdown1,mseccountdown2,mseccountdown3,flashcountdown;

unsigned int totalbytecnt,xoncnt,xoffcnt,decimalpoint;
unsigned char frameerror,dataoverrun,numstrg[9];

extern int _textmode;

/* Static Variables */
unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
unsigned char UART_RxHead;
unsigned char UART_RxTail;
unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
unsigned char UART_TxHead;
unsigned char UART_TxTail;
char  xoffsent, xonsent, enablexonoff, echoinchars,sendxoff;
char  rxcnt,checkflag,rxcntmax;
unsigned char  instrg[8],vlow,vhigh,blankindex,ledindex;

unsigned char FlashAddress[3];
const char signon1[] = "\n\r\n\rLED8 Monitor V0.85\n\r";

// Function declarations
unsigned char EEPROMwrite(int,unsigned char);
unsigned char EEPROMread(int);
int  ledchars[12];


// *******************************************************
//    Start of code
// *******************************************************
int  IncrementNumStrg(int index)
{
 	if(numstrg[index]==NUMBER0)  {
	    numstrg[index] = NUMBER1;
		return 0;
		}
	return index;
}

void Increment_Numstrg()
{
	numstrg[7]++;
	if(numstrg[7]>9)  {
	    numstrg[7] = 0;
		numstrg[6]++;
		if(numstrg[6]>9) {
		    numstrg[6] = 0;
			numstrg[5]++;
			if(numstrg[5]>9) {
			    numstrg[5] = 0;
			    numstrg[4]++;
				if(numstrg[4]>9) {
				    numstrg[4] = 0;
			        numstrg[3]++;
					if(numstrg[3]>9) {
			            numstrg[3] = 0;
			            numstrg[2]++;
						if(numstrg[2]>9) {
			                numstrg[2] = 0;
			                numstrg[1]++;
						    if(numstrg[1]>9) {
			                    numstrg[1] = 0;
			                    numstrg[0]++;
							    if(numstrg[0]>9)  numstrg[0] = 0;
								}
							}
						}
					}
				}
			}
		}
}

// Convert a integer number to string and display
void Display_Int_Number(int ivalue)
{
    unsigned char outstrg[12];
	int i;
	
	decimalpoint = 8;
//	sprintf(outstrg,"%#8d",ivalue);
	itoa(outstrg,ivalue,10);
	for(i=0; i<strlen(outstrg); i++) {
		if(outstrg[i]==' ') numstrg[i] = 10; 
		else numstrg[i] = outstrg[i]-'0';
		} 
}

	//if(decimalpoint == 7) sprintf(outstrg,"%#9.7f",fvalue);
	//else if(decimalpoint == 6) sprintf(outstrg,"%#9.6f",fvalue);
	//else if(decimalpoint == 5) sprintf(outstrg,"%#9.5f",fvalue);
	//else if(decimalpoint == 4) sprintf(outstrg,"%#9.4f",fvalue);
	//else if(decimalpoint == 3) sprintf(outstrg,"%#9.3f",fvalue);
	//else if(decimalpoint == 2) sprintf(outstrg,"%#9.2f",fvalue);
	//else if(decimalpoint == 1) sprintf(outstrg,"%#9.1f",fvalue);

// Convert a floating point number to string and display
//    Set decimal point where you want
void Display_Float_Number(float fvalue,int decimallocation)
{
	char *outstrg;
	int skipcnt,i,status,decimaloffset;
	
	skipcnt = 0;
	outstrg = ftoa(fvalue,&status);
	// Find decimal point location
	for(i=0; i<8; i++)  numstrg[i] = 0;
	decimalpoint = decimallocation;
	for(i=0; i<strlen(outstrg)-1; i++) {
		if(outstrg[i]=='.') {
		    skipcnt = 1;
			decimaloffset = decimallocation - (8-i);
			}
		if(outstrg[i]==' ') numstrg[i] = 10; 
		else numstrg[i+decimaloffset] = outstrg[i+skipcnt+decimaloffset]-'0';
		} 
}

// Initialize LED8 to default starting state
void Init_LED8()
{
    ledchars[0] = NUMBER0;
	ledchars[1] = NUMBER1;
    ledchars[2] = NUMBER2;
	ledchars[3] = NUMBER3;
    ledchars[4] = NUMBER4;
	ledchars[5] = NUMBER5;
    ledchars[6] = NUMBER6;
	ledchars[7] = NUMBER7;
    ledchars[8] = NUMBER8;
	ledchars[9] = NUMBER9;
	ledchars[10]= NUMBER10;
}

unsigned char TWI_Act_On_Failure_In_Last_Transmission(unsigned char TWIerrorMsg)
{
    // A failure has occurred, use TWIerrorMsg to determine the nature of 
	// the failure and take appropriate actions.
    // See header file for a list of possible failures messages.
  
    // This very simple example puts the error code on PORTB and restarts 
	// the transceiver with all the same data in the transmission buffers.
    //PORTB = TWIerrorMsg;
    TWI_Start_Transceiver();
                    
    return TWIerrorMsg; 
}

unsigned char messageBuf[TWI_BUFFER_SIZE],TWI_slaveAddress,temp;

// See if any commands came in
//   The TWI handler places received data in circular buffer so the 
//   main program loop can pick the date up when its ready.  Note 
//   that the ISR is able to respond to data retreival requests
//   immediately as they come in.

// This needs to be redone for multi byte operation
// and to not wait
unsigned char  Check_TWI()
{
    // Check if the TWI Transceiver has completed an operation.
    if(!TWI_Transceiver_Busy())   {
    	// Check if the last operation was successful
      	if(TWI_statusReg.status.lastTransOK)   {
    	    // Check if the last operation was a reception
        	if(TWI_statusReg.status.RxDataInBuf)   {
          		TWI_Get_Data_From_Transceiver(messageBuf, 2);         
    			// Check if the last operation was a reception 
				// as General Call        
          		if(TWI_statusReg.status.genAddressCall) {
          		    // Do operation here                                   
          			}
    			// Ends up here if the last operation was a 
				// reception as Slave Address Match                  
          		else  {
    				// Example of how to interpret a command and respond.
          			// TWI_CMD_MASTER_WRITE stores the data to PORTB
            		if(messageBuf[0]==TWI_CMD_MASTER_WRITE)  ;
                        
            		// TWI_CMD_MASTER_READ prepares the data from PINB 
					// in the transceiver buffer for the TWI master to fetch.
            		if(messageBuf[0]==TWI_CMD_MASTER_READ) {
              			messageBuf[0] = PINB;                             
              			TWI_Start_Transceiver_With_Data(messageBuf,1);           
            			}
          			}
        		}

    	    // Check if the TWI Transceiver has already been started.
    	    // If not then restart it to prepare it for new receptions.             
            if(!TWI_Transceiver_Busy())   {
                TWI_Start_Transceiver();
                }      
            }
        // Ends up here if the last operation completed unsuccessfully
        else {
           TWI_Act_On_Failure_In_Last_Transmission(TWI_Get_State_Info());
           }
       }
	return 'p';
}

//*************************************************************
//  Main work starts here
//*************************************************************
void main()
{
	unsigned char inchar,inbyte,twistatus1,twistatus2,twistatus3,twistatus4;
	unsigned char inbytes[5],toggle,slaverx,firstmeasurement;
	int i,icnt,j,loopsetting,loopmode,blackfinresetmode,countervalue,numbr;
	int dataoutputperiod,ivalue;
	long loopcount,delaynum;
	float xvalue,yvalue;


	Init_AVR();
	Init_LED8();

	loopsetting = 500;   // 1 second delay
	loopmode = 0;  toggle = 0;
	mseccountdown1 = loopsetting;
	loopcount = 0;
	dataoutputperiod = 10;  // seconds
	blankindex = 0;
	
//	WriteEEPROM(0,"SOC Robotics V0.85");

	//Set_LED_Segments(0x55);
	//Decimal_Point(OFF);
	//PORTB |= 0x01;
	
    for(i=0; i<7; i++)  numstrg[i] = 10;
	numstrg[7] = 0;
	numstrg[8] = 0;
	
	delaynum = 0;
	decimalpoint = 5;
	xvalue = 12.24500;
	ivalue = 100;
	i = 0; 

    // Own TWI slave address
    TWI_slaveAddress = 0x10;
	
    // Initialise TWI module for slave operation. 
    // Include address and/or enable General Call.
    TWI_Slave_Initialise((TWI_slaveAddress<<TWI_ADR_BITS)|(TRUE<<TWI_GEN_BIT));                                 //
  
    // Start the TWI transceiver to enable reception of the 
	// first command from the TWI Master.
    TWI_Start_Transceiver();

	_SEI(); // Enable interrupts => enable UART interrupts

	while(1) {
	    for(i=0; i<10; i++)  {
            for(j=0; j<8; j++)  numstrg[j] = i;
	        decimalpoint = i;
			if(decimalpoint>8) decimalpoint = 8;
		    Delaymsec(150);
		    }
		}
	
	// Real work starts here 	
	while (1) {
	
		    // See if anything came in TWI
			Check_TWI();
			
			// Display some sample values
			if(delaynum++>50000) {
			    //Increment_Numstrg();
				//for(i=0; i<10; i++)  numstrg[i] = 10;
				xvalue += .00025;
				//xvalue += 1;
				ivalue += 1;
				
				//for(j=0; j<8; j++) numstrg[j] = 10;
				//numstrg[i++] = '1'-'0';
				//if(i>8) i = 0;
				
				Display_Float_Number(xvalue,5);
				//Display_Int_Number(ivalue);
				delaynum = 0;
				}
			//Update_NumStrg();
			//ShortDelay(delaynum);
			//delaynum += 10;
			//if(delaynum>5000) delaynum = 1;
	
		// Blink green led slowly to show AVR is alive
		if(mseccountdown1==0) {
			//Set_LED_Segments(0x55);
			mseccountdown1 = loopsetting;
			if(loopmode)  {
			    //LED_Power(LED3,ON);
				loopmode = 0;
				}
			else {
			    //LED_Power(LED3,OFF);
				loopmode = 1;
				}
			}
		}
}

/*  
  // A simplified example.
  // This will store data received on PORTB, and increment it before sending it back.

  TWI_Start_Transceiver( );    
         
  for(;;)
  {
    if ( ! TWI_Transceiver_Busy() )                              
    {
      if ( TWI_statusReg.RxDataInBuf )
      {
        TWI_Get_Data_From_Transceiver(&temp, 1);  
        PORTB = temp;
      }
      temp = PORTB + 1;
      TWI_Start_Transceiver_With_Data(&temp, 1); 
    }
    __no_operation();   // Do something else while waiting
  }
}

*/
